local lib = {}


	
function lib.ModifyLine(cs, id, gm, yt, es, eb, s1, s2, f1, f2)
    cs.lines[tostring(id)].mod = gm
	cs.lines[tostring(id)].tab = tonumber(yt)
    cs.lines[tostring(id)].extraString = tostring(es)
    cs.lines[tostring(id)].extraBool = eb
    cs.lines[tostring(id)].string1 = tostring(s1)
	cs.lines[tostring(id)].string2 = tostring(s2)
	cs.lines[tostring(id)].formula1 = tostring(f1)
	cs.lines[tostring(id)].formula2 = tostring(f2)
end

-- clear all lines
function lib.ClearAllLines(cs)
	cs.lines = {}
end

-- returns a string containing the details of the input Dice Result
function lib.DiceResultDetail(dr)
	local s = ""
	-- can be used as a template to parse a dice result table
	for kf,vf in pairs(dr) do	
		s = s .. string.format("%s (%s):\n", vf.formula, vf.result[1].color)
		for kd,vd in pairs(vf.result) do
			-- display all the details
			--s = s .. string.format("%d (%s): %d %s\n", kd, vd.color, vd.value, vd.label)
			
			-- display only the values and colors
			s = s .. string.format("\t%s\n", vd.label)
		end
	end
	return s
end

-- returns the total sum of the input Dice Result
function lib.DiceResultTotal(dr)
	local t = 0
	for kf,vf in pairs(dr) do	
		sign = 1
		-- find sign on formula
		if string.match(vf.formula, "-") then sign = -1 end
		for kd,vd in pairs(vf.result) do
			-- sum all values
			t = t + sign * vd.value
		end
	end	
	return t
end

-- returns a string displaying the whole input Dice Result
function lib.DiceResultFull(dr)
	-- concat details and total
	s = string.format("%s", lib.DiceResultDetail(dr))
	s = s .. string.format("total: %d", lib.DiceResultTotal(dr))
	return s
end

-- returns a table containing the n highest dice from the table t
function lib.GetNHighestDiceIdx(t, n)
	-- create a table that we can sort while keeping the key
	ts = {}
	for k,v in pairs(t) do
		ts[k] = { key = k, value = v.value }
	end
	-- sort the table by value
	table.sort(ts, function(a, b) return a.value<b.value end)
	-- keep the n highest
	for i=1,#ts-n do table.remove(ts,1) end
	-- create the table with only the keys of the two highest
	tr = {}
	for k,v in pairs(ts) do
		tr[k] = v.key
	end
	return tr
end

-- returns a table containing the n highest or lowest (depending on hl) dice from the table t
function lib.GetHighestOrLowestN(t, hl, n)
	-- watch out: t is passed by ref, so the real t is sorted
	table.sort(t, function(v1,v2) return v1.value<v2.value end)
	tr = {}
	local d
	if (hl=="h") then d = #t-n elseif (hl=="l") then d = 0 else return t end
	for i=1,n do tr[i] = t[i + d] end
	return tr
end

return lib
